import os
import json

from celery import Celery
from celery.schedules import crontab
from celery.utils.log import get_task_logger

from .import_tasks.downloader import *
from .import_tasks.extractor import *
from .import_tasks.importer import *
from .import_tasks.extractor_ical import *
from .import_tasks.custom_extractors import *


# Set the default Django settings module for the 'celery' program.
APP_ENV = os.getenv("APP_ENV", "dev")
os.environ.setdefault("DJANGO_SETTINGS_MODULE", f"agenda_culturel.settings.{APP_ENV}")

app = Celery("agenda_culturel")

logger = get_task_logger(__name__)

chromiumDownloader = ChromiumHeadlessDownloader()


# Using a string here means the worker doesn't have to serialize
# the configuration object to child processes.
# - namespace='CELERY' means all celery-related configuration keys
#   should have a `CELERY_` prefix.
app.config_from_object("django.conf:settings", namespace="CELERY")

# Load task modules from all registered Django apps.
app.autodiscover_tasks()


def close_import_task(taskid, success, error_message, importer):
    from agenda_culturel.models import BatchImportation

    task = BatchImportation.objects.get(celery_id=taskid)
    task.status = (
        BatchImportation.STATUS.SUCCESS if success else BatchImportation.STATUS.FAILED
    )
    task.nb_initial = importer.get_nb_events()
    task.nb_imported = importer.get_nb_imported_events()
    task.nb_updated = importer.get_nb_updated_events()
    task.nb_removed = importer.get_nb_removed_events()

    fields = ["status", "nb_initial", "nb_updated", "nb_imported", "nb_removed"]
    if not success:
        task.error_message = error_message
        fields.append("error_message")
    task.save(update_fields=fields)


@app.task(bind=True)
def import_events_from_json(self, json):
    from agenda_culturel.models import BatchImportation
    from .db_importer import DBImporterEvents

    # create a batch importation
    importation = BatchImportation(celery_id=self.request.id)
    # save batch importation
    importation.save()

    logger.info("Import events from json: {}".format(self.request.id))

    importer = DBImporterEvents(self.request.id)

    # try:
    success, error_message = importer.import_events(json)

    # finally, close task
    close_import_task(self.request.id, success, error_message, importer)
    """except Exception as e:
        logger.error(e)
        close_import_task(self.request.id, False, e, importer)"""


@app.task(bind=True)
def run_recurrent_import(self, pk):
    from agenda_culturel.models import RecurrentImport, BatchImportation
    from .db_importer import DBImporterEvents

    logger.info("Run recurrent import: {}".format(self.request.id))

    # get the recurrent import
    rimport = RecurrentImport.objects.get(pk=pk)

    # create a batch importation
    importation = BatchImportation(recurrentImport=rimport, celery_id=self.request.id)
    # save batch importation
    importation.save()

    # create an importer
    importer = DBImporterEvents(self.request.id)

    # prepare downloading and extracting processes
    downloader = (
        SimpleDownloader()
        if rimport.downloader == RecurrentImport.DOWNLOADER.SIMPLE
        else chromiumDownloader
    )
    if rimport.processor == RecurrentImport.PROCESSOR.ICAL:
        extractor = ICALExtractor()
    elif rimport.processor == RecurrentImport.PROCESSOR.ICALNOBUSY:
        extractor = ICALNoBusyExtractor()
    elif rimport.processor == RecurrentImport.PROCESSOR.ICALNOVC:
        extractor = ICALNoVCExtractor()
    elif rimport.processor == RecurrentImport.PROCESSOR.LACOOPE:
        extractor = lacoope.CExtractor()
    elif rimport.processor == RecurrentImport.PROCESSOR.LACOMEDIE:
        extractor = lacomedie.CExtractor()
    elif rimport.processor == RecurrentImport.PROCESSOR.LEFOTOMAT:
        extractor = lefotomat.CExtractor()
    elif rimport.processor == RecurrentImport.PROCESSOR.LAPUCEALOREILLE:
        extractor = lapucealoreille.CExtractor()
    elif rimport.processor == RecurrentImport.PROCESSOR.MECWORDPRESS:
        extractor = wordpress_mec.CExtractor()
    else:
        extractor = None

    if extractor is None:
        logger.error("Unknown extractor")
        close_import_task(self.request.id, False, "Unknown extractor", importer)

    # set parameters
    u2e = URL2Events(downloader, extractor)
    url = rimport.source
    browsable_url = rimport.browsable_url
    category = str(rimport.defaultCategory)
    location = rimport.defaultLocation
    tags = rimport.defaultTags
    published = rimport.defaultPublished

    try:
        # get events from website
        events = u2e.process(
            url,
            browsable_url,
            default_values={"category": category, "location": location, "tags": tags},
            published=published,
        )

        # convert it to json
        json_events = json.dumps(events, default=str)

        # import events (from json)
        success, error_message = importer.import_events(json_events)

        # finally, close task
        close_import_task(self.request.id, success, error_message, importer)
    except Exception as e:
        logger.error(e)
        close_import_task(self.request.id, False, e, importer)


@app.task(bind=True)
def daily_imports(self):
    from agenda_culturel.models import RecurrentImport

    logger.info("Imports quotidiens")
    imports = RecurrentImport.objects.filter(
        recurrence=RecurrentImport.RECURRENCE.DAILY
    )

    for imp in imports:
        run_recurrent_import.delay(imp.pk)


@app.task(bind=True)
def run_all_recurrent_imports(self):
    from agenda_culturel.models import RecurrentImport

    logger.info("Imports complets")
    imports = RecurrentImport.objects.all()

    for imp in imports:
        run_recurrent_import.delay(imp.pk)


@app.task(bind=True)
def weekly_imports(self):
    from agenda_culturel.models import RecurrentImport

    logger.info("Imports hebdomadaires")
    imports = RecurrentImport.objects.filter(
        recurrence=RecurrentImport.RECURRENCE.WEEKLY
    )

    for imp in imports:
        run_recurrent_import.delay(imp.pk)


app.conf.beat_schedule = {
    "daily_imports": {
        "task": "agenda_culturel.celery.daily_imports",
        # Daily imports at 3:14 a.m.
        "schedule": crontab(hour=3, minute=14),
    },
    "weekly_imports": {
        "task": "agenda_culturel.celery.weekly_imports",
        # Daily imports on Mondays at 2:22 a.m.
        "schedule": crontab(hour=2, minute=22, day_of_week="mon"),
    },
}

app.conf.timezone = "Europe/Paris"
