from django import template
from django.utils.safestring import mark_safe
from django.urls import reverse_lazy
from django.template.defaultfilters import pluralize, linebreaks, urlize


from agenda_culturel.models import Event
from django.db.models import Q

from .utils_extra import picto_from_name

register = template.Library()


@register.filter
def in_date(event, date):
    return event.filter(
        (Q(start_day__lte=date) & Q(end_day__gte=date))
        | (Q(end_day=None) & Q(start_day=date))
    )


@register.filter
def can_show_start_time(event, day=None):
    if day is not None and day == event.start_day:
        return True
    return event.start_time and (not event.end_day or event.end_day == event.start_day)


@register.filter
def can_show_end_time(event, day=None):
    if day is not None and day == event.end_day and event.start_day != event.end_day:
        return True
    return False


@register.filter
def need_complete_display(event, display_full=True):
    return (
        event.end_day
        and event.end_day != event.start_day
        and (event.start_time or event.end_time or display_full)
    )


@register.filter
def picto_status(event):
    if event.is_draft():
        return picto_from_name("eye-off", "Brouillon")
    elif event.is_trash():
        return picto_from_name("trash-2", "À la corbeille")
    else:
        return ""


@register.simple_tag
def show_badges_events(placement="top"):
    nb_drafts = Event.nb_draft_events()
    if nb_drafts != 0:
        return mark_safe(
            '<a href="'
            + reverse_lazy("moderation")
            + '?status=draft" class="badge" data-placement="'
            + placement
            + '" data-tooltip="'
            + str(nb_drafts)
            + " brouillon"
            + pluralize(nb_drafts)
            + ' à valider">'
            + picto_from_name("calendar")
            + " "
            + str(nb_drafts)
            + "</a>"
        )
    else:
        return ""


@register.simple_tag
def show_badge_unknown_places(placement="top"):
    nb_unknown = Event.objects.filter(exact_location__isnull=True).count()
    if nb_unknown != 0:
        return mark_safe(
            '<a href="'
            + reverse_lazy("view_unknown_places")
            + '" class="badge" data-placement="'
            + placement
            + '" data-tooltip="'
            + str(nb_unknown)
            + " événement"
            + pluralize(nb_unknown)
            + ' sans lieu défini">'
            + picto_from_name("map-pin")
            + " "
            + str(nb_unknown)
            + "</a>"
        )
    else:
        return ""


@register.simple_tag
def event_field_verbose_name(the_field):
    return Event._meta.get_field(the_field).verbose_name


@register.simple_tag
def field_to_html(field, key):
    if field is None:
        return "-"
    elif key == "description":
        return urlize(mark_safe(linebreaks(field)))
    elif key == "reference_urls":
        return mark_safe(
            "<ul>"
            + "".join(['<li><a href="' + u + '">' + u + "</a></li>" for u in field])
            + "</ul>"
        )
    elif key == "image":
        return mark_safe('<a href="' + field + '">' + field + "</a>")
    elif key == "local_image":
        if field:
            return mark_safe('<img src="' + field.url + '" />')
        else:
            return "-"
    else:
        return field


@register.filter
def add_url_category(url, c):
    if c is None:
        return url
    if url == "":
        return "category=" + str(c.pk)
    else:
        return url + "&category=" + str(c.pk)
